/*
 * sender_access_list plugin script
 */

function rcmail_sender_access_list(prop) {
	if (!rcmail.env.uid && (!rcmail.message_list || !rcmail.message_list.get_selection().length))
		return;

	if (!prop || prop == 'sender_access_list')
		prop = 'blacklist';

	var prev_sel = null;

	// also select childs of (collapsed) threads
	if (rcmail.message_list) {
		if (rcmail.env.uid) {
			if (rcmail.message_list.rows[rcmail.env.uid].has_children && !rcmail.message_list.rows[rcmail.env.uid].expanded) {
				if (!rcmail.message_list.in_selection(rcmail.env.uid)) {
					prev_sel = rcmail.message_list.get_selection();
					rcmail.message_list.select_row(rcmail.env.uid);
				}

				rcmail.message_list.select_childs(rcmail.env.uid);
				rcmail.env.uid = null;
			}
			else if (rcmail.message_list.get_single_selection() == rcmail.env.uid) {
				rcmail.env.uid = null;
			}
		}
		else {
			selection = rcmail.message_list.get_selection();
			for (var i in selection) {
				if (rcmail.message_list.rows[selection[i]].has_children && !rcmail.message_list.rows[selection[i]].expanded)
					rcmail.message_list.select_childs(selection[i]);
			}
		}
	}

	var uids = rcmail.env.uid ? rcmail.env.uid : rcmail.message_list.get_selection().join(',');

	var lock = rcmail.set_busy(true, 'loading');
	rcmail.http_post('plugin.sender_access_list.' + prop, '_uid='+uids+'&_mbox='+urlencode(rcmail.env.mailbox), lock);

	if (prev_sel) {
		rcmail.message_list.clear_selection();

		for (var i in prev_sel)
			rcmail.message_list.select_row(prev_sel[i], CONTROL_KEY);
	}
}

function rcmail_sender_access_list_blacklist(prop) {
	rcmail_sender_access_list('blacklist');
}

function rcmail_sender_access_list_whitelist(prop) {
	rcmail_sender_access_list('whitelist');
}

rcube_webmail.prototype.rcmail_sender_access_list_move = function(mbox, uid) {
	var prev_uid = rcmail.env.uid;
	var prev_sel = null;
	var a_uids = uid.split(",");

	if (rcmail.message_list && a_uids.length == 1 && !rcmail.message_list.rows[a_uids[0]]) {
		rcmail.env.uid = a_uids[0];
	}
	else if (rcmail.message_list && a_uids.length == 1 && !rcmail.message_list.in_selection(a_uids[0]) && !rcmail.env.threading) {
		rcmail.env.uid = a_uids[0];
		rcmail.message_list.remove_row(rcmail.env.uid, false);
	}
	else if (rcmail.message_list && (!rcmail.message_list.in_selection(a_uids[0]) || a_uids.length != rcmail.message_list.selection.length)) {
		prev_sel = rcmail.message_list.get_selection();
		rcmail.message_list.clear_selection();

		for (var i in a_uids)
			rcmail.message_list.select_row(a_uids[i], CONTROL_KEY);
	}

	if (mbox)
		rcmail.move_messages(mbox);
	else
		rcmail.delete_messages();

	rcmail.env.uid = prev_uid;

	if (prev_sel) {
		rcmail.message_list.clear_selection();

		for (var i in prev_sel) {
			if (prev_sel[i] != uid)
				rcmail.message_list.select_row(prev_sel[i], CONTROL_KEY);
		}
	}
}

function rcmail_sender_access_list_init() {
	if (rcmail.env.sender_access_list_markmenu && window.rcm_contextmenu_register_command) {
		rcm_contextmenu_register_command('senderblacklist', 'rcmail_sender_access_list_blacklist', rcmail.gettext('sender_access_list.menublacklist'), 'reply', null, true, null, null, 'markmessage senderblacklist');
		rcm_contextmenu_register_command('senderwhitelist', 'rcmail_sender_access_list_whitelist', rcmail.gettext('sender_access_list.menuwhitelist'), 'reply', null, true, null, null, 'markmessage senderwhitelist');
		$('#rcmContextMenu li.unflagged').removeClass('separator_below');
		$('#rcmContextMenu li.reply').addClass('separator_above');
	}
}

function rcmail_sender_access_list_update() {
/*
	var spamobj = $('#' + rcmail.buttons['plugin.sender_access_list.blacklist'][0].id);
	var hamobj = $('#' + rcmail.buttons['plugin.sender_access_list.whitelist'][0].id);

	if (spamobj.parent('li').length > 0) {
		spamobj = spamobj.parent();
		hamobj = hamobj.parent();
	}

	if (!rcmail.env.sender_access_list_override && rcmail.env.junk_mailbox && rcmail.env.mailbox != rcmail.env.junk_mailbox) {
		$('#rcmContextMenu li.sender_access_list').show();
		$('#rcmContextMenu li.senderwhitelist').hide();
		spamobj.show();
		hamobj.hide();
	}
	else if (!rcmail.env.sender_access_list_override) {
		$('#rcmContextMenu li.sender_access_list').hide();
		$('#rcmContextMenu li.senderwhitelist').show();
		spamobj.hide();
		hamobj.show();
	}
*/
}

function rcmail_sender_access_list_status(command) {
	switch (command) {
		case 'beforedelete':
			if (!rcmail.env.flag_for_deletion && rcmail.env.trash_mailbox &&
				rcmail.env.mailbox != rcmail.env.trash_mailbox &&
				(rcmail.message_list && !rcmail.message_list.shiftkey))
				rcmail.enable_command('plugin.sender_access_list.blacklist', 'plugin.sender_access_list.whitelist', false);

			break;
		case 'beforemove':
		case 'beforemoveto':
			rcmail.enable_command('plugin.sender_access_list.blacklist', 'plugin.sender_access_list.whitelist', false);
			break;
		case 'aftermove':
		case 'aftermoveto':
			if (rcmail.env.action == 'show')
				rcmail.enable_command('plugin.sender_access_list.blacklist', 'plugin.sender_access_list.whitelist', true);

			break;
		case 'afterpurge':
		case 'afterexpunge':
			if (!rcmail.env.messagecount && rcmail.task == 'mail')
				rcmail.enable_command('plugin.sender_access_list.blacklist', 'plugin.sender_access_list.whitelist', false);

			break;
	}
}

$(document).ready(function() {
	if (window.rcmail) {
		rcmail.addEventListener('init', function(evt) {
			// register command (directly enable in message view mode)
			rcmail.register_command('plugin.sender_access_list.blacklist', rcmail_sender_access_list_blacklist, rcmail.env.uid);
			rcmail.register_command('plugin.sender_access_list.whitelist', rcmail_sender_access_list_whitelist, rcmail.env.uid);

			if (rcmail.message_list) {
				rcmail.message_list.addEventListener('select', function(list) {
					rcmail.enable_command('plugin.sender_access_list.blacklist', list.get_selection().length > 0);
					rcmail.enable_command('plugin.sender_access_list.whitelist', list.get_selection().length > 0);
				});
			}
		});

		rcmail.add_onload('rcmail_sender_access_list_init()');
		rcmail.addEventListener('listupdate', function(props) { rcmail_sender_access_list_update(); } );

		// update button activation after external events
		rcmail.addEventListener('beforedelete', function(props) { rcmail_sender_access_list_status('beforedelete'); } );
		rcmail.addEventListener('beforemove', function(props) { rcmail_sender_access_list_status('beforemove'); } );
		rcmail.addEventListener('beforemoveto', function(props) { rcmail_sender_access_list_status('beforemoveto'); } );
		rcmail.addEventListener('aftermove', function(props) { rcmail_sender_access_list_status('aftermove'); } );
		rcmail.addEventListener('aftermoveto', function(props) { rcmail_sender_access_list_status('aftermoveto'); } );
		rcmail.addEventListener('afterpurge', function(props) { rcmail_sender_access_list_status('afterpurge'); } );
		rcmail.addEventListener('afterexpunge', function(props) { rcmail_sender_access_list_status('afterexpunge'); } );
	}
});